==> oer-0.0.1-20190731.023331-5-sources.jar: com/xdja/pki/itsca/oer/cert/TbsCertBuilder.java <==
package com.xdja.pki.itsca.oer.cert;

import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.utils.ByteArrayUtils;
import com.xdja.pki.itsca.oer.utils.TimeUtils;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.security.PublicKey;
import java.security.interfaces.ECPublicKey;

public class TbsCertBuilder {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SequenceOfItsAidList itsAidList;
    private SequenceOfItsAidSspList itsAidSspList;

    private GeographicRegion geographicRegion;

    public void setItsAidList(SequenceOfItsAidList itsAidList) {
        this.itsAidList = itsAidList;
    }

    public void setItsAidSspList(SequenceOfItsAidSspList itsAidSspList) {
        this.itsAidSspList = itsAidSspList;
    }

    public void setGeographicRegion(GeographicRegion geographicRegion) {
        this.geographicRegion = geographicRegion;
    }

    public TbsCert build(PublicKey verifyPublicKey, PublicKey encPublicKey, String subjectName, Enumerated.Value subjectTypeValue) throws IOException {
        TbsCert tbsCert = new TbsCert();

        buildSubjectInfo(tbsCert, subjectName, subjectTypeValue);
        buildSubjectAttribute(tbsCert, verifyPublicKey, encPublicKey);
        buildValidityRestriction(tbsCert);

        return tbsCert;
    }

    public void buildSubjectInfo(TbsCert tbsCert, String subjectName, Enumerated.Value subjectTypeValue) throws IOException {
        ByteArrayUtils.printHexBinary(2, "SubjectInfo SEQUENCE", null);
        SubjectInfo subjectInfo = new SubjectInfo();

        ByteArrayUtils.printHexBinary(3, "SubjectType ENUMERATED [" + subjectTypeValue.getIndex() + " " + subjectTypeValue.getDesc() + "]", null);
        SubjectType subjectType = new SubjectType(subjectTypeValue);
        subjectInfo.setSubjectType(subjectType);
        ByteArrayUtils.printHexBinary(3, "SubjectType", subjectInfo.getEncode());

        ByteArrayUtils.printHexBinary(3, "SubjectName OCTET STRING [" + subjectName + "]", null);
        byte[] name = subjectName.getBytes();
        ByteArrayUtils.printHexBinary(3, "SubjectName", name);
        subjectInfo.setSubjectName(name);

        ByteArrayUtils.printHexBinary(2, "SubjectInfo", subjectInfo.getEncode());
        tbsCert.setSubjectInfo(subjectInfo);
    }

    public void buildSubjectAttribute(TbsCert tbsCert, PublicKey verifyPublicKey, PublicKey encPublicKey) throws IOException {
        ByteArrayUtils.printHexBinary(2, "SubjectAttribute SEQUENCE", null);
        SubjectAttribute subjectAttribute = new SubjectAttribute();

        ByteArrayUtils.printHexBinary(3, "PublicVerifyKey SEQUENCE", null);
        PublicVerifyKey verifyKey = new PublicVerifyKey();

        Enumerated.Value sgdSm2 = EccCurve.SGD_SM2;
        ByteArrayUtils.printHexBinary(4, "EccCurve ENUMERATED [" + sgdSm2.getIndex() + " " + sgdSm2.getDesc() + "]", null);
        EccCurve eccCurve = new EccCurve(sgdSm2);
        verifyKey.setEccCurve(eccCurve);
        ByteArrayUtils.printHexBinary(4, "EccCurve", eccCurve.getEncode());

        ByteArrayUtils.printHexBinary(4, "EccPoint CHOICE [index = 4] ", null);
        EccPoint eccPoint = new EccPoint();

        ByteArrayUtils.printHexBinary(5, "Uncompressed SEQUENCE", null);
        Uncompressed uncompressed = new Uncompressed();
        ECPublicKey ecPublicKey = (ECPublicKey) verifyPublicKey;
        byte[] x = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineX());
        byte[] y = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineY());
        String xx = Hex.toHexString(x);
        int xLength = xx.length();
        if (xx.length() < 64) {
            for (int i = xLength; i < 64; i++) {
                xx = "0" + xx;
            }
        }
        x = Hex.decode(xx);
        String yy = Hex.toHexString(y);
        int yLength = yy.length();
        if (yy.length() < 64) {
            for (int i = yLength; i < 64; i++) {
                yy = "0" + yy;
            }
        }
        y = Hex.decode(yy);
        ByteArrayUtils.printHexBinary(5, "x OCTET STRING", null);
        ByteArrayUtils.printHexBinary(5, "x", x);
        uncompressed.setX(x);
        ByteArrayUtils.printHexBinary(5, "y OCTET STRING", null);
        ByteArrayUtils.printHexBinary(5, "y", y);
        uncompressed.setY(y);
        ByteArrayUtils.printHexBinary(5, "Uncompressed", uncompressed.getEncode());
        eccPoint.setUncompressed(uncompressed);
        verifyKey.setEccPoint(eccPoint);
        ByteArrayUtils.printHexBinary(4, "EccPoint", eccPoint.getEncode());
        subjectAttribute.setVerifyKey(verifyKey);
        ByteArrayUtils.printHexBinary(3, "PublicVerifyKey", verifyKey.getEncode());
        if (null != encPublicKey) {
            PublicEncryptionKey encryptionKey = new PublicEncryptionKey();
            encryptionKey.setSupportedSymmAlg(new SymmetricAlgorithm(SymmetricAlgorithm.SGD_SM4_ECB));
            encryptionKey.setEccCurve(new EccCurve(EccCurve.SGD_SM2));
            uncompressed = new Uncompressed();
            ECPublicKey encKey = (ECPublicKey) encPublicKey;
            x = BigIntegers.asUnsignedByteArray(encKey.getW().getAffineX());
            y = BigIntegers.asUnsignedByteArray(encKey.getW().getAffineY());
            xx = Hex.toHexString(x);
            xLength = xx.length();
            if (xx.length() < 64) {
                for (int i = xLength; i < 64; i++) {
                    xx = "0" + xx;
                }
            }
            x = Hex.decode(xx);
            yy = Hex.toHexString(y);
            yLength = yy.length();
            if (yy.length() < 64) {
                for (int i = yLength; i < 64; i++) {
                    yy = "0" + yy;
                }
            }
            y = Hex.decode(yy);
            ByteArrayUtils.printHexBinary(5, "x OCTET STRING", null);
            ByteArrayUtils.printHexBinary(5, "x", x);
            uncompressed.setX(x);
            ByteArrayUtils.printHexBinary(5, "y OCTET STRING", null);
            ByteArrayUtils.printHexBinary(5, "y", y);
            uncompressed.setY(y);
            eccPoint = new EccPoint();
            eccPoint.setUncompressed(uncompressed);
            encryptionKey.setPublicKey(eccPoint);
            subjectAttribute.setEncryptionKey(encryptionKey);
        }
        byte[] assure = new byte[]{0};
        subjectAttribute.setAssuranceLevel(new SubjectAssurance(assure));
        if (itsAidList != null) {
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidList SEQUENCE OF ItsAid", null);
            subjectAttribute.setItsAidList(itsAidList);
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidList", itsAidList.getEncode());
        }
        if (itsAidSspList != null) {
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidSspList SEQUENCE OF ItsAidSsp", null);
            subjectAttribute.setItsSspList(itsAidSspList);
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidSspList", itsAidSspList.getEncode());
        }

        tbsCert.setSubjectAttribute(subjectAttribute);
        ByteArrayUtils.printHexBinary(2, "SubjectAttribute", subjectAttribute.getEncode());
    }

    public void buildValidityRestriction(TbsCert tbsCert) throws IOException {
        ByteArrayUtils.printHexBinary(2, "ValidityRestriction SEQUENCE", null);
        ValidityRestriction validityRestriction = new ValidityRestriction();
        ByteArrayUtils.printHexBinary(3, "ValidityPeriod CHOICE [index=0]", null);
        ValidityPeriod validityPeriod = new ValidityPeriod();

        ByteArrayUtils.printHexBinary(4, "Time32 INTEGER", null);
        TimeStartAndEnd startAndEnd = new TimeStartAndEnd();

        long nowTime = TimeUtils.getNowTime();
        Time32 time32 = new Time32(nowTime);
        startAndEnd.setStartValidity(time32);
        ByteArrayUtils.printHexBinary(4, "Time32 start", time32.getEncode());
        long timeEnd = TimeUtils.getTimeAfterYear(1);
        time32 = new Time32(timeEnd);
        startAndEnd.setEndValidity(time32);
        validityPeriod.setTimeStartAndEnd(startAndEnd);
        ByteArrayUtils.printHexBinary(4, "Time32 end ", time32.getEncode());
        ByteArrayUtils.printHexBinary(3, "ValidityPeriod", validityPeriod.getEncode());
        validityRestriction.setValidityPeriod(validityPeriod);

        if (geographicRegion != null) {
            ByteArrayUtils.printHexBinary(3, "GeographicRegion CHOICE [index=]", validityPeriod.getEncode());
            validityRestriction.setGeographicRegion(geographicRegion);
            ByteArrayUtils.printHexBinary(3, "GeographicRegion", validityPeriod.getEncode());
        }

        tbsCert.setValidityRestriction(validityRestriction);
        ByteArrayUtils.printHexBinary(2, "ValidityRestriction", validityRestriction.getEncode());
    }

}

==> oer-0.0.2-20190819.074155-10-sources.jar: com/xdja/pki/itsca/oer/cert/TbsCertBuilder.java <==
package com.xdja.pki.itsca.oer.cert;

import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Enumerated;
import com.xdja.pki.itsca.oer.utils.ByteArrayUtils;
import com.xdja.pki.itsca.oer.utils.TimeUtils;
import org.bouncycastle.util.BigIntegers;
import org.bouncycastle.util.encoders.Hex;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.security.PublicKey;
import java.security.interfaces.ECPublicKey;

public class TbsCertBuilder {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private SequenceOfItsAidList itsAidList;
    private SequenceOfItsAidSspList itsAidSspList;

    private GeographicRegion geographicRegion;

    private Time32 startTime;
    private Time32 endTime;

    public void setStartTime(Time32 startTime) {
        this.startTime = startTime;
    }

    public void setEndTime(Time32 endTime) {
        this.endTime = endTime;
    }

    public void setItsAidList(SequenceOfItsAidList itsAidList) {
        this.itsAidList = itsAidList;
    }

    public void setItsAidSspList(SequenceOfItsAidSspList itsAidSspList) {
        this.itsAidSspList = itsAidSspList;
    }

    public void setGeographicRegion(GeographicRegion geographicRegion) {
        this.geographicRegion = geographicRegion;
    }

    public TbsCert build(PublicKey verifyPublicKey, PublicKey encPublicKey, String subjectName, Enumerated.Value subjectTypeValue) throws IOException {
        TbsCert tbsCert = new TbsCert();

        buildSubjectInfo(tbsCert, subjectName, subjectTypeValue);
        buildSubjectAttribute(tbsCert, verifyPublicKey, encPublicKey);
        buildValidityRestriction(tbsCert);

        return tbsCert;
    }

    public void buildSubjectInfo(TbsCert tbsCert, String subjectName, Enumerated.Value subjectTypeValue) throws IOException {
        ByteArrayUtils.printHexBinary(2, "SubjectInfo SEQUENCE", null);
        SubjectInfo subjectInfo = new SubjectInfo();

        ByteArrayUtils.printHexBinary(3, "SubjectType ENUMERATED [" + subjectTypeValue.getIndex() + " " + subjectTypeValue.getDesc() + "]", null);
        SubjectType subjectType = new SubjectType(subjectTypeValue);
        subjectInfo.setSubjectType(subjectType);
        ByteArrayUtils.printHexBinary(3, "SubjectType", subjectInfo.getEncode());

        ByteArrayUtils.printHexBinary(3, "SubjectName OCTET STRING [" + subjectName + "]", null);
        byte[] name = subjectName.getBytes();
        ByteArrayUtils.printHexBinary(3, "SubjectName", name);
        subjectInfo.setSubjectName(name);

        ByteArrayUtils.printHexBinary(2, "SubjectInfo", subjectInfo.getEncode());
        tbsCert.setSubjectInfo(subjectInfo);
    }

    public void buildSubjectAttribute(TbsCert tbsCert, PublicKey verifyPublicKey, PublicKey encPublicKey) throws IOException {
        ByteArrayUtils.printHexBinary(2, "SubjectAttribute SEQUENCE", null);
        SubjectAttribute subjectAttribute = new SubjectAttribute();

        ByteArrayUtils.printHexBinary(3, "PublicVerifyKey SEQUENCE", null);
        PublicVerifyKey verifyKey = new PublicVerifyKey();

        Enumerated.Value sgdSm2 = EccCurve.SGD_SM2;
        ByteArrayUtils.printHexBinary(4, "EccCurve ENUMERATED [" + sgdSm2.getIndex() + " " + sgdSm2.getDesc() + "]", null);
        EccCurve eccCurve = new EccCurve(sgdSm2);
        verifyKey.setEccCurve(eccCurve);
        ByteArrayUtils.printHexBinary(4, "EccCurve", eccCurve.getEncode());

        ByteArrayUtils.printHexBinary(4, "EccPoint CHOICE [index = 4] ", null);
        EccPoint eccPoint = new EccPoint();

        ByteArrayUtils.printHexBinary(5, "Uncompressed SEQUENCE", null);
        Uncompressed uncompressed = new Uncompressed();
        ECPublicKey ecPublicKey = (ECPublicKey) verifyPublicKey;
        byte[] x = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineX());
        byte[] y = BigIntegers.asUnsignedByteArray(ecPublicKey.getW().getAffineY());
        String xx = Hex.toHexString(x);
        int xLength = xx.length();
        if (xx.length() < 64) {
            for (int i = xLength; i < 64; i++) {
                xx = "0" + xx;
            }
        }
        x = Hex.decode(xx);
        String yy = Hex.toHexString(y);
        int yLength = yy.length();
        if (yy.length() < 64) {
            for (int i = yLength; i < 64; i++) {
                yy = "0" + yy;
            }
        }
        y = Hex.decode(yy);
        ByteArrayUtils.printHexBinary(5, "x OCTET STRING", null);
        ByteArrayUtils.printHexBinary(5, "x", x);
        uncompressed.setX(x);
        ByteArrayUtils.printHexBinary(5, "y OCTET STRING", null);
        ByteArrayUtils.printHexBinary(5, "y", y);
        uncompressed.setY(y);
        ByteArrayUtils.printHexBinary(5, "Uncompressed", uncompressed.getEncode());
        eccPoint.setUncompressed(uncompressed);
        verifyKey.setEccPoint(eccPoint);
        ByteArrayUtils.printHexBinary(4, "EccPoint", eccPoint.getEncode());
        subjectAttribute.setVerifyKey(verifyKey);
        ByteArrayUtils.printHexBinary(3, "PublicVerifyKey", verifyKey.getEncode());
        if (null != encPublicKey) {
            PublicEncryptionKey encryptionKey = new PublicEncryptionKey();
            encryptionKey.setSupportedSymmAlg(new SymmetricAlgorithm(SymmetricAlgorithm.SGD_SM4_ECB));
            encryptionKey.setEccCurve(new EccCurve(EccCurve.SGD_SM2));
            uncompressed = new Uncompressed();
            ECPublicKey encKey = (ECPublicKey) encPublicKey;
            x = BigIntegers.asUnsignedByteArray(encKey.getW().getAffineX());
            y = BigIntegers.asUnsignedByteArray(encKey.getW().getAffineY());
            xx = Hex.toHexString(x);
            xLength = xx.length();
            if (xx.length() < 64) {
                for (int i = xLength; i < 64; i++) {
                    xx = "0" + xx;
                }
            }
            x = Hex.decode(xx);
            yy = Hex.toHexString(y);
            yLength = yy.length();
            if (yy.length() < 64) {
                for (int i = yLength; i < 64; i++) {
                    yy = "0" + yy;
                }
            }
            y = Hex.decode(yy);
            ByteArrayUtils.printHexBinary(5, "x OCTET STRING", null);
            ByteArrayUtils.printHexBinary(5, "x", x);
            uncompressed.setX(x);
            ByteArrayUtils.printHexBinary(5, "y OCTET STRING", null);
            ByteArrayUtils.printHexBinary(5, "y", y);
            uncompressed.setY(y);
            eccPoint = new EccPoint();
            eccPoint.setUncompressed(uncompressed);
            encryptionKey.setPublicKey(eccPoint);
            subjectAttribute.setEncryptionKey(encryptionKey);
        }
        byte[] assure = new byte[]{0};
        subjectAttribute.setAssuranceLevel(new SubjectAssurance(assure));
        if (itsAidList != null) {
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidList SEQUENCE OF ItsAid", null);
            subjectAttribute.setItsAidList(itsAidList);
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidList", itsAidList.getEncode());
        }
        if (itsAidSspList != null) {
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidSspList SEQUENCE OF ItsAidSsp", null);
            subjectAttribute.setItsSspList(itsAidSspList);
            ByteArrayUtils.printHexBinary(3, "SequenceOfItsAidSspList", itsAidSspList.getEncode());
        }

        tbsCert.setSubjectAttribute(subjectAttribute);
        ByteArrayUtils.printHexBinary(2, "SubjectAttribute", subjectAttribute.getEncode());
    }

    public void buildValidityRestriction(TbsCert tbsCert) throws IOException {
        ByteArrayUtils.printHexBinary(2, "ValidityRestriction SEQUENCE", null);
        ValidityRestriction validityRestriction = new ValidityRestriction();
        ByteArrayUtils.printHexBinary(3, "ValidityPeriod CHOICE [index=0]", null);
        ValidityPeriod validityPeriod = new ValidityPeriod();

        ByteArrayUtils.printHexBinary(4, "Time32 INTEGER", null);
        TimeStartAndEnd startAndEnd = new TimeStartAndEnd();
        if (null == endTime){
            long nowTime = TimeUtils.getNowTime();
            Time32 time32 = new Time32(nowTime);
            startAndEnd.setStartValidity(time32);
            ByteArrayUtils.printHexBinary(4, "Time32 start", time32.getEncode());
            long timeEnd = TimeUtils.getTimeAfterYear(1);
            time32 = new Time32(timeEnd);
            startAndEnd.setEndValidity(time32);
            validityPeriod.setTimeStartAndEnd(startAndEnd);
            ByteArrayUtils.printHexBinary(4, "Time32 end ", time32.getEncode());
        }else {
            if (null == startTime){
                validityPeriod.setTimeEnd(endTime);
            }else {
                startAndEnd = new TimeStartAndEnd();
                startAndEnd.setStartValidity(startTime);
                startAndEnd.setEndValidity(endTime);
                validityPeriod.setTimeStartAndEnd(startAndEnd);
            }
        }
        ByteArrayUtils.printHexBinary(3, "ValidityPeriod", validityPeriod.getEncode());
        validityRestriction.setValidityPeriod(validityPeriod);

        if (geographicRegion != null) {
            ByteArrayUtils.printHexBinary(3, "GeographicRegion CHOICE [index=]", validityPeriod.getEncode());
            validityRestriction.setGeographicRegion(geographicRegion);
            ByteArrayUtils.printHexBinary(3, "GeographicRegion", validityPeriod.getEncode());
        }

        tbsCert.setValidityRestriction(validityRestriction);
        ByteArrayUtils.printHexBinary(2, "ValidityRestriction", validityRestriction.getEncode());
    }

}
